package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.model.platform.CommonTreeModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 通用内存树查询服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 11:16
 * @Modify :
 **/
public interface ICoreMemTreeService {

    /**
     * List 分装成树数据
     * @param list      原始List
     * @param parentId  父节点ID
     * @return 多棵树
     */
    @ApiOperation(value = "分装岗位树",notes = "分装岗位树")
    List<SysPosition> packTreeSysPosition(List<SysPosition> list,Long parentId);

    /**
     * 根据用户岗位关系 修剪树的数据
     * @param list      分装好的树
     * @param refList   需要保留的书关系
     * @return 修剪后的树
     */
    @ApiOperation(value = "分装岗位树",notes = "根据用户岗位关系分装岗位树")
    List<SysPosition> packTreeSysPositionBySysRefUserPosition(List<SysPosition> list, List<SysRefUserPosition> refList);

    /**
     * 根据岗位ID 修剪树的数据
     * @param list  分装好的树
     * @param ids   需要保留的书关系
     * @return  修剪后的树
     */
    @ApiOperation(value = "分装岗位树",notes = "根据制定IDS分装岗位树")
    List<SysPosition> packTreeSysPositionByIds(List<SysPosition> list, List<Long> ids);

    /**
     * 根据岗位对象分装书数据
     * @param list
     * @param id
     * @return
     */
    @ApiOperation(value = "分装岗位树",notes = "根据制定ID分装岗位树")
    SysPosition packTreeSysPositionById(List<SysPosition> list, Long id);

    /**
     * 获取制定树的所有叶子节点ID
     * @param position
     * @return
     */
    @ApiOperation(value = "查询岗位子节点",notes = "查询字节点岗位ID")
    List<Long> findChildSysPosition(SysPosition position);

    @ApiOperation(value = "查询岗位子节点",notes = "查询字节点岗位ID")
    List<Long> findSysPositionChildIdList(Long id);

    /********************************************* Start SysOrganize Tree *********************************************/

    @ApiOperation(value = "分装组织树",notes = "分装组织树")
    List<SysOrganize> packTreeSysOrganize(List<SysOrganize> list,Long parentId);

    @ApiOperation(value = "分装组织树",notes = "根据用户组织关系分装组织树")
    List<SysOrganize> packTreeSysOrganizeBySysRefUserOrganize(List<SysOrganize> list, List<SysRefUserDepartment> refList);

    @ApiOperation(value = "分装组织树",notes = "根据用户组织关系分装组织树")
    List<SysOrganize> packTreeSysOrganizeBySysRefUserInfoOrganize(List<SysOrganize> list, List<SysRefUserInfoDepartment> refList);

    @ApiOperation(value = "分装组织树",notes = "根据制定IDS分装组织树")
    List<SysOrganize> packTreeSysOrganizeByIds(List<SysOrganize> list, List<Long> ids);

    @ApiOperation(value = "分装组织树",notes = "根据制定ID分装组织树")
    SysOrganize packTreeSysOrganizeById(List<SysOrganize> list, Long id);

    @ApiOperation(value = "找到相应字节点",notes = "从一棵树中找到相应的子节点")
    SysOrganize getTreeSysOrganizeById(List<SysOrganize> list, Long id);

    @ApiOperation(value = "查询组织子节点",notes = "查询字节点组织ID")
    List<Long> findChildSysOrganize(SysOrganize organize);

    @ApiOperation(value = "查询组织子节点",notes = "查询字节点组织ID")
    List<Long> findSysOrganizeChildIdList(Long id);

    /********************************************* End SysOrganize Tree *********************************************/

    /********************************************* Start CommonTreeModel Tree *********************************************/

    @ApiOperation(value = "组织树转TreeModel",notes = "根据帐号ID 分装TreeModel")
    List<CommonTreeModel> packTreeTreeModelByRefUserDepartment(List<SysOrganize> organizeList, List<SysDepartment> departmentList, List<SysRefUserDepartment> refList);

    @ApiOperation(value = "组织树转TreeModel",notes = "根据帐号ID 分装TreeModel")
    List<CommonTreeModel> packTreeTreeModelRoot();

    @ApiOperation(value = "组织树转TreeModel",notes = "根据帐号ID 分装TreeModel")
    List<CommonTreeModel> packTreeTreeModelByUserId(Long userId);

    @ApiOperation(value = "组织树转TreeModel",notes = "使用分装好的树转为TreeMode")
    List<CommonTreeModel> packTreeTreeModelSysOrganize(List<SysOrganize> list);

    @ApiOperation(value = "组织树转TreeModel",notes = "使用分装好的树转为TreeMode")
    List<CommonTreeModel> packTreeTreeModelSysDepartment(List<SysDepartment> list);

    @ApiOperation(value = "组织树转TreeModel",notes = "使用分装好的树转为TreeMode")
    List<CommonTreeModel> packTreeTreeModel(List<CommonTreeModel> list, List<SysDepartment> departmentList);

    @ApiOperation(value = "获取部门ID 集合",notes = "使用分装好的树转为TreeMode")
    List<Long> findDepartmentIdListByOrganizeIdAndDepartmentIdAndUserId(Long organizeId, Long departmentId, Long userId);

    List<Long> findDepartmentIdListByOrganizeIdAndDepartmentId(Long organizeId, Long departmentId);

    /********************************************* End CommonTreeModel Tree *********************************************/


    /********************************************* Start SysDepartment Tree *********************************************/
    @ApiOperation(value = "分装部门树",notes = "分装部门树")
    List<SysDepartment> packTreeSysDepartment(List<SysDepartment> list,Long parentId);

    @ApiOperation(value = "分装部门树",notes = "根据用户部门关系分装部门树")
    List<SysDepartment> packTreeSysDepartmentBySysRefUserOrganize(List<SysDepartment> list, List<SysRefUserDepartment> refList);

    @ApiOperation(value = "分装部门树",notes = "根据用户部门关系分装部门树")
    List<SysDepartment> packTreeSysDepartmentBySysRefUserInfoOrganize(List<SysDepartment> list, List<SysRefUserInfoDepartment> refList);

    @ApiOperation(value = "分装部门树",notes = "根据制定IDS分装部门树")
    List<SysDepartment> packTreeSysDepartmentByIds(List<SysDepartment> list, List<Long> ids);

    @ApiOperation(value = "分装部门树",notes = "根据制定ID分装部门树")
    SysDepartment packTreeSysDepartmentById(List<SysDepartment> list, Long id);

    @ApiOperation(value = "查询部门子节点",notes = "查询字节点部门ID")
    List<Long> findChildSysDepartment(SysDepartment organize);

    @ApiOperation(value = "查询部门子节点",notes = "查询字节点部门ID")
    List<Long> findSysDepartmentChildIdList(Long id);

    /********************************************* End SysDepartment Tree *********************************************/

    /********************************************* Start SysMenu Tree *********************************************/
    @ApiOperation(value = "分装功能树",notes = "分装功能树")
    List<SysMenu> packTreeSysMenu(List<SysMenu> list,Long parentId);

    @ApiOperation(value = "分装功能树",notes = "根据用户功能关系分装功能树")
    List<SysMenu> packTreeSysMenuBySysRefRoleMenu(List<SysMenu> list, List<SysRefRoleMenu> refList);

    @ApiOperation(value = "分装功能树",notes = "根据制定IDS分装功能树")
    List<SysMenu> packTreeSysMenuByIds(List<SysMenu> list, List<Long> ids);

    @ApiOperation(value = "分装功能树",notes = "根据制定ID分装功能树")
    SysMenu packTreeSysMenuById(List<SysMenu> list, Long id);

    @ApiOperation(value = "查询功能子节点",notes = "查询字节点功能ID")
    List<Long> findChildSysMenu(SysMenu organize);

    @ApiOperation(value = "查询功能子节点",notes = "根据用户ID 查询功能树")
    List<SysMenu> packTreeSysMenuByUserId(Long userId);

    /**
     * 根据用户ID 和 节点ID 查询功能树
     * @param userId
     * @param parentId
     * @return
     */
    @ApiOperation(value = "查询功能子节点",notes = "根据用户ID 和 节点ID 查询功能树")
    List<SysMenu> packTreeSysMenuByUserIdAndParentId(Long userId,Long parentId);

    @ApiOperation(value = "查询功能子节点",notes = "根据父节点ID 和 菜单类型 查询功能树")
    List<SysMenu> packTreeSysMenuByParentIdAndMenuType(List<SysMenu> list,Long parentId,Integer menuType);

    @ApiOperation(value = "查询功能子节点",notes = "根据用户ID 查询功能树")
    List<SysMenu> packTreeSysMenu();

    /********************************************* End SysMenu Tree *********************************************/

}
