package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ILicenseClickService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysDepartmentService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysOrganizeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPositionService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.core.apiservice.util.SysUserPasswordUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.license.ImppLicense;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.model.platform.UserDetailPagerModel;
import cn.estsh.i3plus.pojo.model.platform.UserRolePagerModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.SysUserPasswordRepository;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.license.ImppLicenseTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-24 14:02
 * @Modify:
 **/
@RestController
@Api(tags = "系统用户管理服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/personnel")
public class PersonnelController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(PersonnelController.class);

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysFileService fileService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private ILicenseClickService licenseClickService;

    @Autowired
    private ISysOrganizeService organizeService;

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private ISysDepartmentService departmentService;

    @Autowired
    private SysUserPasswordUtil userPasswordUtil;

    @Autowired
    private SysUserPasswordRepository userPasswordRDao;

    /**
     * 添加用户信息
     *
     * @param model 用户信息
     * @return 处理结果
     */
    @PostMapping(value = "/user-detail/insert")
    @ApiOperation(value = "添加用户信息", notes = "返回内容添加用户信息")
    public ResultBean insertUserDetailModel(UserDetailModel model) {
        try {
            startMultiService();
            licenseClickService.checkLicenseNumberUser();

            SysUser user = model.getSysUser();
            SysUserInfo info = model.getSysUserInfo();

            checkUserDetailModel(model, false);
            userPasswordUtil.checkPasswordSave(user);
            personnelService.checkSysUserOnly(user);

            if (StringUtils.isBlank(info.getUserJoinDate())) {
                info.setUserJoinDate(TimeTool.getToday());
            }

            info.setUserInfoStatus(CommonEnumUtil.USER_INFO_STATUS.INCUMBENT.getValue());
            info.setUserLoginNum(0);
            info.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
            ConvertBean.modelInitialize(info, AuthUtil.getSessionUser());

            user.setUserType(CommonEnumUtil.USER_TYPE.USER.getValue());
            user.setUserStatus(CommonEnumUtil.USER_STATUS.ENABLE.getValue());
            user.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
            user.setUserLoginNum(0);
            ConvertBean.modelInitialize(user, AuthUtil.getSessionUser());

            SysDepartment infoDepartment = departmentService.get(Long.parseLong(model.getInfoDepartmentId()));
            ValidatorBean.checkNotNull(infoDepartment, "用户主部门不存在");
            SysOrganize infoOrganize = organizeService.get(infoDepartment.getOrganizeId());
            ValidatorBean.checkNotNull(infoOrganize, "用户主组织不存在");
            SysPosition position = positionService.get(Long.parseLong(model.getInfoPositionId()));
            ValidatorBean.checkNotNull(position, "用户主岗位不存在");

            SysDepartment userDepartment = departmentService.get(Long.parseLong(model.getUserDepartmentId()));
            ValidatorBean.checkNotNull(userDepartment, "账号主部门不存在");
            SysOrganize userOrganize = organizeService.get(userDepartment.getOrganizeId());

            info.setDepartmentId(infoDepartment.getId());
            info.setDepartmentNameRdd(infoDepartment.getName());
            info.setOrganizeId(infoOrganize.getId());
            info.setOrganizeNameRdd(infoOrganize.getName());
            info.setOrganizeCode(infoOrganize.getOrganizeCode());
            info.setPositionId(position.getId());
            info.setPositionNameRdd(position.getName());
            personnelService.saveSysUserInfo(info);

            user.setDepartmentId(userDepartment.getId());
            user.setDepartmentNameRdd(userDepartment.getName());
            user.setOrganizeId(userOrganize.getId());
            user.setOrganizeNameRdd(userOrganize.getName());
            user.setOrganizeCode(userOrganize.getOrganizeCode());
            user.setUserInfoId(info.getId());
            //松下需要创建的新用户 第一次登陆系统就要重置密码
//            user.setUserPasswordLastModifyTime(TimeTool.getNowTime(true));
            refreshSysUserPassword(user);
            SysUser userSaved=personnelService.saveSysUser(user);

            // 添加保存passwordId进SysUser表
            SysUserPassword pwd = new SysUserPassword();
            pwd.setUserId(userSaved.getId());
            pwd.setUserPassword(userSaved.getUserLoginPassword());
            ConvertBean.serviceModelInitialize(pwd, userSaved.getUserName());
            SysUserPassword savedPassword = userPasswordRDao.save(pwd);
            user.setUserLoginPasswordId(savedPassword.getId());
            personnelService.saveSysUser(user);

            info.setUserId(user.getId());
            personnelService.saveSysUserInfo(info);

            // 关系 刷新
            refreshRef(user, info, model);

            endMultiService();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改用户信息
     *
     * @param model 用户信息
     * @return 处理结果
     */
    @PutMapping(value = "/user-detail/update")
    @ApiOperation(value = "修改用户信息", notes = "返回内容添加用户信息")
    public ResultBean updateUserDetailModel(UserDetailModel model) {
        try {
            startMultiService();

            SysUser user = model.getSysUser();
            SysUserInfo info = model.getSysUserInfo();

            checkUserDetailModel(model, true);
            personnelService.checkSysUserOnly(user);

            SysUserInfo userInfo = personnelService.getSysUserInfoById(info.getId());
            SysUser sysUser = personnelService.getSysUserById(user.getId());

            SysDepartment infoDepartment = departmentService.get(Long.parseLong(model.getInfoDepartmentId()));
            ValidatorBean.checkNotNull(infoDepartment, "用户主部门不存在");
            SysOrganize infoOrganize = organizeService.get(infoDepartment.getOrganizeId());
            ValidatorBean.checkNotNull(infoOrganize, "用户主组织不存在");
            SysPosition position = positionService.get(Long.parseLong(model.getInfoPositionId()));
            ValidatorBean.checkNotNull(position, "用户主岗位不存在");

            SysDepartment userDepartment = departmentService.get(Long.parseLong(model.getUserDepartmentId()));
            ValidatorBean.checkNotNull(userDepartment, "账号主部门不存在");

            SysOrganize userOrganize = organizeService.get(userDepartment.getOrganizeId());
            ValidatorBean.checkNotNull(userOrganize, "账号主组织不存在");

            userInfo.setDepartmentId(infoDepartment.getId());
            userInfo.setDepartmentNameRdd(infoDepartment.getName());
            userInfo.setOrganizeId(infoOrganize.getId());
            userInfo.setOrganizeNameRdd(infoOrganize.getName());
            userInfo.setOrganizeCode(infoOrganize.getOrganizeCode());
            userInfo.setPositionId(position.getId());
            userInfo.setPositionNameRdd(position.getName());
            userInfo.setName(info.getName());
            userInfo.setUserEmpNo(info.getUserEmpNo());
            userInfo.setUserJoinDate(info.getUserJoinDate());
            userInfo.setUserResignationDate(info.getUserResignationDate());
            userInfo.setUserCardType(info.getUserCardType());
            userInfo.setUserCardNumber(info.getUserCardNumber());
            userInfo.setUserSchool(info.getUserSchool());
            userInfo.setUserSchoolProfession(info.getUserSchoolProfession());
            userInfo.setUserSchoolLeaveDate(info.getUserSchoolLeaveDate());
            userInfo.setUserSchoolEducation(info.getUserSchoolEducation());
            userInfo.setUserSex(info.getUserSex());
            userInfo.setUserBirthplace(info.getUserBirthplace());
            userInfo.setUserRace(info.getUserRace());
            userInfo.setUserMarriage(info.getUserMarriage());
            userInfo.setUserEmergencyContact(info.getUserEmergencyContact());
            userInfo.setUserEmergencyPhone(info.getUserEmergencyPhone());
            userInfo.setUserFamilyAddress(info.getUserFamilyAddress());
            userInfo.setUserBornDate(info.getUserBornDate());
            userInfo.setUserHobby(info.getUserHobby());
            userInfo.setUserPoliticalStatus(info.getUserPoliticalStatus());
            userInfo.setUserGrade(info.getUserGrade());
            userInfo.setUserEmployeeType(info.getUserEmployeeType());
            userInfo.setUserImageId(info.getUserImageId());

            sysUser.setUserLoginName(user.getUserLoginName());
            sysUser.setLanguageCode(user.getLanguageCode());
            sysUser.setUserEmail(user.getUserEmail());
            sysUser.setUserPhone(user.getUserPhone());
            sysUser.setDepartmentId(userDepartment.getId());
            sysUser.setDepartmentNameRdd(userDepartment.getName());
            sysUser.setOrganizeId(userOrganize.getId());
            sysUser.setOrganizeNameRdd(userOrganize.getName());
            sysUser.setUserName(userInfo.getName());
            sysUser.setUserEmpNo(userInfo.getUserEmpNo());

            // 关系 刷新
            refreshRef(sysUser, userInfo, model);

            personnelService.saveSysUser(sysUser);
            personnelService.saveSysUserInfo(userInfo);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改用户信息
     *
     * @param model 用户信息
     * @return 处理结果
     */
    @PutMapping(value = "/user/update")
    @ApiOperation(value = "修改用户信息", notes = "返回内容添加用户信息")
    public ResultBean updateUserModel(UserDetailModel model) {
        try {
            checkUserModel(model);
            SysUser user = personnelService.getSysUserById(Long.parseLong(model.getUserId()));
            initUser(model, user);
            personnelService.checkSysUserOnly(user);

            SysDepartment userDepartment = departmentService.get(Long.parseLong(model.getUserDepartmentId()));
            ValidatorBean.checkNotNull(userDepartment, "账号主部门不存在");
            SysOrganize userOrganize = organizeService.get(userDepartment.getOrganizeId());

            user.setDepartmentId(userDepartment.getId());
            user.setDepartmentNameRdd(userDepartment.getName());
            user.setOrganizeId(userOrganize.getId());
            user.setOrganizeNameRdd(userOrganize.getName());

            // 关系 刷新
            refreshRef(user, null, model);
            personnelService.saveSysUser(user);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取用户信息
     *
     * @param id 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/user-info/get-detail/{id}")
    @ApiOperation(value = "获取用户信息", notes = "查询根据ID查询用户相关所有信息")
    public ResultBean getUserInfoById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");
            Map<String, Object> resultMap = new HashMap<>();


            SysUserInfo info = personnelService.getSysUserInfoById(Long.parseLong(id));
            ValidatorBean.checkNotNull(info, "用户不存在");
            SysUser user = personnelService.getSysUserById(info.getUserId());
            ValidatorBean.checkNotNull(user, "账号不存在");

            List<SysRefUserInfoDepartment> refUserInfoDepartmentList = personnelService.findSysRefUserInfoDepartmentById(info.getId());
            List<SysRefUserPosition> refUserPositionList = personnelService.findSysRefUserPositionByUserInfoId(info.getId());
            List<SysRefUserDepartment> refUserDepartmentList = personnelService.findSysRefUserDepartmentByUserId(user.getId());
            List<SysRefUserRole> refUserRoleList = personnelService.findSysRefUserRole(user.getId());
            SysFile image = info.getUserImageId() != null ? fileService.getSysFileById(info.getUserImageId()) : null;

            resultMap.put("user", user);
            resultMap.put("refUserInfoDepartmentList", refUserInfoDepartmentList);
            resultMap.put("refUserPositionList", refUserPositionList);
            resultMap.put("refUserDepartmentList", refUserDepartmentList);
            resultMap.put("refUserRoleList", refUserRoleList);
            resultMap.put("image", image);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resultMap);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取账号信息
     *
     * @param id 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/user/get-detail/{id}")
    @ApiOperation(value = "获取账号信息", notes = "查询根据ID查询账号相关所有信息")
    public ResultBean getUserById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");
            Map<String, Object> resultMap = new HashMap<>();

            SysUser user = personnelService.getSysUserById(Long.parseLong(id));

            List<SysRefUserDepartment> refUserInfoDepartmentList = personnelService.findSysRefUserDepartmentByUserId(user.getId());
            List<SysRefUserRole> refUserRoleList = personnelService.findSysRefUserRole(user.getId());

            resultMap.put("refUserDepartmentList", refUserInfoDepartmentList);
            resultMap.put("refUserRoleList", refUserRoleList);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resultMap);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 删除用户信息
     *
     * @param id 用户id
     * @return 处理结果
     */
    @DeleteMapping(value = "/user-info/delete/{id}")
    @ApiOperation(value = "删除用户信息", notes = "根据用户信息id删除信息")
    public ResultBean deleteSysUserInfo(@PathVariable("id") String id) {
        try {
            startMultiService();

            //条件验证
            ValidatorBean.checkNotNull(id, "id为空");

            SysUserInfo info = personnelService.getSysUserInfoById(Long.parseLong(id));
            if (info != null) {
                personnelService.deleteSysUserInfo(new Long[]{info.getId()}, getSessionUser().getUserName());
                personnelService.deleteSysUser(new Long[]{info.getUserId()}, getSessionUser().getUserName());

                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     *
     * @param ids id数组
     * @return 处理
     */
    @DeleteMapping(value = "/user-info/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysUserInfoByIds(String[] ids) {
        try {
            startMultiService();

            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户信息。")
                        .build();
            }

            List<SysUserInfo> list = personnelService.findSysUserInfoByIds(StringTool.getArrayLong(ids));

            if (list != null && list.size() > 0) {
                Long[] infoIds = new Long[list.size()];
                Long[] userIds = new Long[list.size()];

                for (int i = 0; i < list.size(); i++) {
                    infoIds[i] = list.get(i).getId();
                    userIds[i] = list.get(i).getUserId();
                }

                personnelService.deleteSysUserInfo(infoIds, getSessionUser().getUserName());
                personnelService.deleteSysUser(userIds, getSessionUser().getUserName());

                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除账号信息
     *
     * @param id 用户账号id
     * @return 处理结果
     */
    @DeleteMapping(value = "/user/delete/{id}")
    @ApiOperation(value = "删除账号信息", notes = "根据账号信息id删除信息")
    public ResultBean deleteSysUser(@PathVariable("id") String id) {
        try {
            startMultiService();

            //条件验证
            ValidatorBean.checkNotNull(id, "id为空");

            SysUser user = personnelService.getSysUserById(Long.parseLong(id));
            if (user != null) {
                personnelService.deleteSysUserInfo(new Long[]{user.getUserInfoId()}, getSessionUser().getUserName());
                personnelService.deleteSysUser(new Long[]{user.getId()}, getSessionUser().getUserName());

                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    /**
     * 根据id批量删除
     *
     * @param ids ids数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/user/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysUserByIds(String[] ids) {
        try {
            startMultiService();

            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户信息。")
                        .build();
            }

            List<SysUser> list = personnelService.findSysUserByIds(StringTool.getArrayLong(ids));

            if (list != null && list.size() > 0) {
                Long[] infoIds = new Long[list.size()];
                Long[] userIds = new Long[list.size()];

                for (int i = 0; i < list.size(); i++) {
                    infoIds[i] = list.get(i).getUserInfoId();
                    userIds[i] = list.get(i).getId();
                }

                personnelService.deleteSysUserInfo(infoIds, getSessionUser().getUserName());
                personnelService.deleteSysUser(userIds, getSessionUser().getUserName());

                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param model 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/query-ref-department")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean queryRefDepartment(@RequestBody UserDetailModel model) {
        try {
            List<Long> idList = new ArrayList<>();

            List<SysRefUserDepartment> refs = personnelService.querySysRefUserDepartmentByUserDetailModel(model);
            if (refs != null && refs.size() > 0) {
                for (SysRefUserDepartment ref : refs) {
                    if (!idList.contains(ref.getUserId())) {
                        idList.add(ref.getUserId());
                    }
                }
            }

            List<SysUser> result = personnelService.querySysUserByUserDetailModelOrIds(model, idList);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param model 处理消息
     * @return 处理结果
     */
    @PostMapping(value = "/pager/query-ref-department")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean queryPagerRefDepartment(@RequestBody UserDetailPagerModel model) {
        try {
            List<Long> idList = new ArrayList<>();
            List<SysRefUserDepartment> refs = personnelService.querySysRefUserDepartmentByUserDetailModel(model.getUserDetailModel());
            if (refs != null && refs.size() > 0) {
                for (SysRefUserDepartment ref : refs) {
                    idList.add(ref.getUserId());
                }
            }

            ListPager<SysUser> result = personnelService.querySysUserPagerByUserDetailModelOrIds(model.getUserDetailModel(), model.getPager(), idList);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(result);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据条件分页查询用户角色信息
     *
     * @param model 条件
     * @return 处理结果
     */
    @PostMapping(value = "/role/get-role-list")
    @ApiOperation(value = "根据条件分页查询用户角色信息", notes = "根据条件分页查询用户角色信息")
    public ResultBean queryPagerByRoleInfo(@RequestBody UserRolePagerModel model) {
        try {
            ListPager<SysRole> listPager = sysRoleService.findPager(model.getSysRole(), model.getPager());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 根据条件分页查询用户信息
     *
     * @param model 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/pager/get-user-info-list")
    @ApiOperation(value = "根据条件分页查询用户信息", notes = "根据条件分页查询用户信息")
    public ResultBean queryPagerByUserInfo(@RequestBody UserDetailPagerModel model) {
        try {
            ListPager<SysUserInfo> listPager = personnelService.querySysUserInfo(model.getUserDetailModel(), model.getPager());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 根据条件分页查询用户信息
     *
     * @param model 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/pager/get-user-list")
    @ApiOperation(value = "根据条件分页查询账号信息", notes = "根据条件分页查询账号信息")
    public ResultBean queryPagerByUser(@RequestBody UserDetailPagerModel model) {
        try {
            ListPager<SysUser> listPager = personnelService.querySysUser(model.getUserDetailModel(), model.getPager());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 获取账号的所有组织信息集合
     *
     * @return 组织信息集合
     */
    @GetMapping(value = "/user/get-organize-list")
    @ApiOperation(value = "获取账号信息", notes = "获取用户账号的所有组织信息")
    public ResultBean getUserOrganizeList() {
        try {
            List<SysOrganize> list = AuthUtil.getSessionUser().getUser().getOrganizeList();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/user/session-organize/{id}")
    @ApiOperation(value = "修改用户会话组织信息", notes = "修改用户会话组织信息")
    public ResultBean updateUserOrganizeList(@PathVariable("id") String idStr) {
        try {
            ValidatorBean.checkIsNumber(idStr, "id为空");
            List<SysOrganize> list = AuthUtil.getSessionUser().getUser().getOrganizeList();
            Long id = Long.parseLong(idStr);
            SysOrganize organize = null;
            for (SysOrganize org : list) {
                if (org.getId().equals(id)) {
                    organize = org;
                }
            }

            if (organize == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("不存在的组织信息")
                        .setErrorSolution("请重新操作")
                        .build();
            }

            // 会话工厂信息保存进Redis
            AuthUtil.setOrganize(organize);
            personnelService.updateSysUserOrganize(AuthUtil.getSessionUser().getUser().getId(), organize.getId());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(organize);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 数据必填校验
     *
     * @param model     用户信息
     * @param isCheckId 是否校验
     * @throws Exception
     */
    public void checkUserDetailModel(UserDetailModel model, boolean isCheckId) throws Exception {
        if (isCheckId) {
            ValidatorBean.beginValid(model)
                    .notNull("userInfoId", model.getUserInfoId())
                    .notNull("userId", model.getUserId());
        } else {
            ValidatorBean.beginValid(model).notNull("userLoginPassword", model.getUserLoginPassword());
        }

        ValidatorBean.beginValid(model)
                .notNull("infoName", model.getInfoName())
                .notNull("infoEmpNo", model.getInfoEmpNo())
                .notNull("infoPositionId", model.getInfoPositionId())
                .notNull("infoDepartmentId", model.getInfoDepartmentId())
                .notNull("userDepartmentId", model.getUserDepartmentId())
                .notNull("userLoginName", model.getUserLoginName());
                // FIXME 松下移除邮箱校验
//                .notNull("userEmail", model.getUserEmail())
                // FIXME 松下移除手机号校验
//               .notNull("userPhone", model.getUserPhone());
    }

    /**
     * 数据必填校验
     *
     * @param model 数据消息
     * @throws Exception
     */
    public void checkUserModel(UserDetailModel model) throws Exception {
        ValidatorBean.beginValid(model)
                .notNull("userInfoId", model.getUserInfoId())
                .notNull("userId", model.getUserId())
                .notNull("infoName", model.getInfoName())
                .notNull("infoEmpNo", model.getInfoEmpNo())
                .notNull("userDepartmentId", model.getUserDepartmentId())
                .notNull("userLoginName", model.getUserLoginName());
    }

    /**
     * 用户信息初始化
     *
     * @param model 用户明细信息
     * @param user  用户信息
     * @throws Exception
     */
    public void initUser(UserDetailModel model, SysUser user) throws Exception {
        if (user != null && model != null) {
            user.setUserLoginName(model.getUserLoginName());
            user.setDepartmentId(Long.parseLong(model.getUserDepartmentId()));
            user.setUserEmail(model.getUserEmail());
            user.setUserPhone(model.getUserPhone());
            user.setLanguageCode(model.getUserLanguageCode());

            if (!StringUtils.isAnyBlank(model.getUserLoginPassword())) {   // 使用新密码
                user.setUserLoginPassword(EncryptTool.hexMD5(model.getUserLoginPassword()));
            }
        }
    }

    /**
     * 关系刷新
     *
     * @param user  账户
     * @param info  用户信息
     * @param model 用户模型
     */
    public void refreshRef(SysUser user, SysUserInfo info, UserDetailModel model) {
        startMultiService();

        if (user != null) {
            // 刷新账号 部门关系
            personnelService.refreshRefSysUserDepartment(user.getId(), StringTool.getArrayLong(model.getUserDepartmentIdList()));
            // 刷新账号 角色关系
            personnelService.refreshRefSysUserRole(user.getId(), StringTool.getArrayLong(model.getUserRoleIdList()));
        }

        if (info != null) {
            // 刷新用户 部门关系
            personnelService.refreshUpdateRefSysUserInfoDepartment(info.getId(), StringTool.getArrayLong(model.getInfoDepartmentIdList()));
            // 刷新用户 岗位关系
            personnelService.refreshRefSysUserInfoPosition(info.getId(), StringTool.getArrayLong(model.getInfoPositionIdList()));
        }
    }

    /**
     * 更新用户密码
     *
     * @param user
     */
    public void refreshSysUserPassword(SysUser user) throws Exception {
        if (user != null) {
            if (user.getId() != null) { // 更新密码
                if (!StringUtils.isAnyBlank(user.getUserLoginPassword())) {   // 使用新密码
                    user.setUserLoginPassword(EncryptTool.hexMD5(user.getUserLoginPassword().trim()));
                } else {  //使用旧密码
                    SysUser sysUser = personnelService.getSysUserById(user.getId());
                    user.setUserLoginPassword(sysUser.getUserLoginPassword());
                }
            } else { // 创建密码
                user.setUserLoginPassword(EncryptTool.hexMD5(user.getUserLoginPassword().trim()));
            }
        }
    }

    /**
     * 根据组织code查询对应的组织信息
     *
     * @param code 组织代码
     * @return 处理结果
     */
    @GetMapping(value = "/organize/get-code")
    @ApiOperation(value = "根据组织code查询对应的组织信息", notes = "根据组织code查询对应的组织信息")
    public ResultBean<SysOrganize> getSysOrganizeByCode(String code) {
        try {
            SysOrganize org = new SysOrganize();
            org.setOrganizeCode(code);
            SysOrganize sysOrganize = organizeService.get(org);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysOrganize);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有组织信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/organize/list")
    @ApiOperation(value = "查询所有组织信息", notes = "查询所有组织信息")
    public ResultBean<SysOrganize> findSysOrganizeList() {
        try {
            List<SysOrganize> sysOrganize = organizeService.findAll();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(sysOrganize);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有组织信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/license")
    @ApiOperation(value = "获取授权信息")
    public ResultBean<ImppLicense> getLicense() {
        try {
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(ImppLicenseTool.getLicense());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
