package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.IDataSeparatorService;
import cn.estsh.i3plus.core.apiservice.util.strategy.IDataSeparatorStrategy;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorMessage;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorRule;
import cn.estsh.i3plus.pojo.platform.repository.DataSeparatorRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.google.common.base.Strings;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2021/12/3 13:44
 * @Modify:
 **/
@Service
public class DataSeparatorServiceImpl implements IDataSeparatorService {

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Resource
    private DataSeparatorRepository dataSeparatorRepository;

    @Override
    public void doSeparate(BaseRepository baseRepository, DataSeparatorMessage msg) {
        //获取数据
        Long id = msg.getId();
        Object bean = baseRepository.getById(id);
        //获取策略
        String strategy = ImppEnumUtil.DATA_SEPARATOR_STRATEGY.codeOfStrategyName(msg.getClassification());
        IDataSeparatorStrategy separator = (IDataSeparatorStrategy) SpringContextsUtil.getBean(strategy);
        //迁移数据
        separator.execute(bean, msg);
        //删除原数据
        baseRepository.deleteById(id);
    }

    @Override
    public boolean addRule(DataSeparatorRule msg) {
        ConvertBean.saveOrUpdate(msg, AuthUtil.getSessionUser().getUserName());
        DataSeparatorRule rule = dataSeparatorRepository.insert(msg);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + ":" + rule.getRefBeanName().toUpperCase();
        redisRes.putObject(redisKey, rule, -1);
        return true;
    }

    @Override
    public boolean deleteRule(Long id) {
        DataSeparatorRule rule = dataSeparatorRepository.getById(id);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + rule.getRefBeanName().toUpperCase();
        dataSeparatorRepository.deleteWeaklyById(id, AuthUtil.getSessionUser().getUserName());
        redisRes.deleteKey(redisKey);
        return true;
    }

    @Override
    public boolean update(DataSeparatorRule msg) {
        ConvertBean.saveOrUpdate(msg, AuthUtil.getSessionUser().getUserName());
        dataSeparatorRepository.update(msg);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + ":" + msg.getRefBeanName().toUpperCase();
        redisRes.putObject(redisKey, msg, -1);
        return true;
    }

    @Override
    public ListPager<DataSeparatorRule> query(DataSeparatorRule rule, Pager pager) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(rule.getOrganizeCode());
        if (Strings.isNullOrEmpty(rule.getClassification())) {
            DdlPreparedPack.getStringEqualPack(rule.getClassification(), "classification", ddlPackBean);
        }

        if (Strings.isNullOrEmpty(rule.getRefBeanName())) {
            DdlPreparedPack.getStringLeftLikerPack(rule.getRefBeanName(), "refBeanName", ddlPackBean);
        }

        if (Strings.isNullOrEmpty(rule.getDatabaseName())) {
            DdlPreparedPack.getStringLeftLikerPack(rule.getDatabaseName(), "databaseName", ddlPackBean);
        }

        if (Strings.isNullOrEmpty(rule.getRuleColumn())) {
            DdlPreparedPack.getStringLeftLikerPack(rule.getRuleColumn(), "ruleColumn", ddlPackBean);
        }
        pager = PagerHelper.getPager(pager, dataSeparatorRepository.findByHqlWhereCount(ddlPackBean));

        return new ListPager(dataSeparatorRepository.findByHqlPage(ddlPackBean, pager), pager);
    }

    @Override
    public void separatorRedisInit() {
        //启动加载所有的数据到redis中
        List<DataSeparatorRule> list = dataSeparatorRepository.list();
        list.forEach(item -> {
            String refBeanName = item.getRefBeanName().toUpperCase();
            String redisKey = CommonConstWords.RES_DATA_SEPARATOR + refBeanName;
            redisRes.putObject(redisKey, item, -1);
        });
    }
}
