package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.apiservice.util.strategy.IDataSeparatorStrategy;
import cn.estsh.i3plus.platform.plugin.datasource.DynamicDataSourceProxy;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2021/12/3 14:02
 * @Modify:
 **/
@Component
@Slf4j
public class DataSeparatorStrategyMysql implements IDataSeparatorStrategy {

    /**
     * 根据目标数据数据的url+数据库名作为key 缓存动态数据源
     */
    public static final Map<String, DynamicDataSourceProxy> DYNAMIC_DATA_SOURCE_PROXY_MAP = new HashMap<>();


    @Override
    public void execute(Object bean, DataSeparatorMessage msg) {
        //获取动态数据源
        DynamicDataSourceProxy dynamicDataSource = getDynamicDataSource(msg);

        //封装Map<columnName,value>
        HashMap<String, Object> objMap = getObjMap(bean);

        //执行sql
        String insertSQL = dynamicDataSource.packInsertSQL(msg.getDestTableName(), objMap);
        try {
            dynamicDataSource.execute(insertSQL);
        } catch (SQLException e) {
            log.error("执行sql:{}失败，失败原因:{}", insertSQL, e.getMessage());
        }


    }

    /**
     * 获取动态数据源
     *
     * @param msg
     * @return
     */
    private DynamicDataSourceProxy getDynamicDataSource(DataSeparatorMessage msg) {
        DynamicDataSourceProxy dataSourceProxy;
        String destUrl = msg.getDestUrl();
        String dataBaseName = msg.getDatabaseName();
        String mapKey = destUrl + ":" + dataBaseName;
        if (!DYNAMIC_DATA_SOURCE_PROXY_MAP.containsKey(mapKey)) {
            String url = "jdbc:mysql://" + destUrl + "/" + dataBaseName + "?autoReconnect=true&useSSL=false&characterEncoding=utf-8";
            dataSourceProxy = DynamicDataSourceProxy.initDataSourceFactory("com.mysql.jdbc.Driver",url,msg.getUserName(),msg.getPassword());
            DYNAMIC_DATA_SOURCE_PROXY_MAP.put(mapKey, dataSourceProxy);
        }
        dataSourceProxy = DYNAMIC_DATA_SOURCE_PROXY_MAP.get(mapKey);
        return dataSourceProxy;
    }

    /**
     * 获取封装的k-v值
     *
     * @param bean
     * @return
     */
    private HashMap<String, Object> getObjMap(Object bean) {
        Field[] declaredFields = bean.getClass().getDeclaredFields();
        Field[] parentFields = bean.getClass().getSuperclass().getDeclaredFields();
        HashMap<String, Object> objMap = new HashMap<>(declaredFields.length);
        List<Field[]> list = new ArrayList<>();
        list.add(declaredFields);
        list.add(parentFields);
        list.forEach(item -> {
            for (Field field : item) {
                field.setAccessible(true);
                if (null != field.getAnnotation(Column.class)) {
                    Column annotation = field.getAnnotation(Column.class);
                    String columnName = annotation.name();
                    try {
                        Object value = field.get(bean);
                        objMap.put(columnName, value);
                    } catch (IllegalAccessException e) {
                        log.error("冷热分离Mysql封装数据错误:{}!", columnName);
                    }
                }
            }
        });
        return objMap;
    }
}
