package cn.estsh.i3plus.core.apiservice.serviceimpl.base.login.strategy;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginStrategyService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseThreadLocal;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.SysLoginModel;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysConfig;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.ExpiredCredentialsException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.function.BiFunction;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.DEFAULT_LANGUAGE;

/**
 * @author Wynne.Lu
 * @date 2021/1/18 下午4:39
 * @desc
 */
@Slf4j
@Service
public class DefaultLoginStrategy implements ISystemLoginStrategyService {

    @Resource(name = "redisCore")
    protected ImppRedis redisCore;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISystemLoginService loginService;

    @Override
    public BiFunction<HttpServletRequest, SysLoginModel, ResultBean> login() {
        return (request, loginModel) -> {
            ResultBean result;
            ValidatorBean.checkNotNull(loginModel.getLanguageCode(), "语言不能为空");
            // TODO 后期移除，暂时用于避免自动登录后前台没有正确的传输组织代码信息
            if ("null".equals(loginModel.getLanguageCode())) {
                loginModel.setLanguageCode(DEFAULT_LANGUAGE);
            }

            // 设置语言代码
            BaseThreadLocal.setData(BaseThreadLocal.LANGUAGE_CODE, loginModel.getLanguageCode());

            ValidatorBean.checkNotNull(loginModel.getLoginName(), "用户名不能为空");
            ValidatorBean.checkNotNull(loginModel.getLoginPwd(), "密码不能为空");

            int sessionMode = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_SESSION_MODE, CommonConstWords.CONFIG_SESSION_MODE_DEFAULT);
            if (sessionMode != CommonEnumUtil.SESSION_MODE.SEIZE.getValue()) {
                AuthUtil.logout();
            }
            // 已经对用户名和密码做过验证了
            // 获取系统参数【密码过期时间】 进行密码有效时间判断
            userService.validatePasswordByLoginNameAndReturnUser(loginModel.getLoginName(),loginModel.getLoginPwd());
            validatePasswordIsExpired(loginModel.getLoginName());

            SessionUser user = userService.loginUser(
                    loginModel.getLoginName().trim(),
                    loginModel.getLoginPwd().trim(),
                    loginModel.getLanguageCode(),
                    loginService.getLoginPlatform(request).getValue(),
                    loginModel.getDeviceId()
            );

            String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getUser().getId();
            redisCore.deleteKey(redisKey);

            result = new ResultBean(true, AuthUtil.getSession().getId().toString(), AuthUtil.getSessionUser());
            result.setUrl("/");

            AuthUtil.setOrganize(user.getUser().getOrganize());
            return result;
        };
    }

    private void validatePasswordIsExpired(String loginName) {
        SysConfig passwordExpireSwitch = RedisCacheTool.getSysConfigByConfigCode(CommonConstWords.CONFIG_PWD_EXPIRE_SWITCH);
        if (passwordExpireSwitch != null
                && CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == Integer.parseInt(passwordExpireSwitch.getConfigValue())) {
            SysConfig passwordExpireDays = RedisCacheTool.getSysConfigByConfigCode(CommonConstWords.CONFIG_PWD_EXPIRE_DAY_TIME);
            if (passwordExpireDays != null && Integer.parseInt(passwordExpireDays.getConfigValue()) > 0) {
                SysUser user = userService.getSysUserByLoginName(loginName);
                if (StringUtils.isEmpty(user.getUserPasswordLastModifyTime())) {
                    throw new ExpiredCredentialsException("用户密码过期 请重新设置新密码");
                } else {
                    DateFormat df = new SimpleDateFormat(CommonConstWords.DATE_TIME_FORMAT_HH_MM_SS_RISK);
                    Date lastModifyDate = null;
                    try {
                        lastModifyDate = df.parse(user.getUserPasswordLastModifyTime());
                    } catch (ParseException e) {
                        throw new ImppBusiException("验证用户密码登录时间，时间解析错误");
                    }
                    long dayGap = (System.currentTimeMillis() - lastModifyDate.getTime()) / (60 * 60 * 1000 * 24);
                    if ((int) dayGap >= Integer.parseInt(passwordExpireDays.getConfigValue())) {
                        throw new ExpiredCredentialsException("用户密码过期 请重新设置新密码");
                    }
                }
            }
        }
    }
}
