package cn.estsh.i3plus.core.apiservice.serviceimpl.base.login.strategy;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginStrategyService;
import cn.estsh.i3plus.core.api.iservice.busi.ILicenseClickService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.tool.ServletRequestTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseThreadLocal;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.SysLoginModel;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.function.BiFunction;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.DEFAULT_LANGUAGE;

/**
 * @author Wynne.Lu
 * @date 2021/1/18 上午10:53
 * @desc
 */
@Lazy
@Slf4j
@Service
public class E9LoginStrategy implements ISystemLoginStrategyService {

    @Resource(name = "redisCore")
    protected ImppRedis redisCore;

    @Autowired
    private ILicenseClickService licenseClickService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISystemLoginService loginService;

    @Override
    public BiFunction<HttpServletRequest, SysLoginModel, ResultBean> login() {
        return (request, loginModel) -> {
            boolean isRequestFromSpecificIp = validateHost(request);
            if (!isRequestFromSpecificIp) {
                return ResultBean.fail("单点登录请求未从指定授权服务器发送，校验失败。");
            }
            licenseClickService.checkLicenseNumberLogin();

            // TODO 后期移除，暂时用于避免自动登录后前台没有正确的传输组织代码信息
            if ("null".equals(loginModel.getLanguageCode())|| StringUtils.isEmpty(loginModel.getLanguageCode())) {
                loginModel.setLanguageCode(DEFAULT_LANGUAGE);
            }

            // 设置语言代码
            BaseThreadLocal.setData(BaseThreadLocal.LANGUAGE_CODE, loginModel.getLanguageCode());
            BaseThreadLocal.setData(PlatformConstWords.AUTH_LOGIN_STRATEGY, ImppEnumUtil.AUTH_LOGIN_STRATEGY.E9.getCode());

            int sessionMode = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_SESSION_MODE, CommonConstWords.CONFIG_SESSION_MODE_DEFAULT);
            if (sessionMode != CommonEnumUtil.SESSION_MODE.SEIZE.getValue()) {
                AuthUtil.logout();
            }

            SessionUser user = userService.loginUser(
                    loginModel.getLoginName().trim(),
                    loginModel.getLoginName().trim(),
                    loginModel.getLanguageCode(),
                    loginService.getLoginPlatform(request).getValue(),
                    loginModel.getDeviceId()
            );

            String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getUser().getId();
            redisCore.deleteKey(redisKey);

            ResultBean result = new ResultBean(true, AuthUtil.getSession().getId().toString(), AuthUtil.getSessionUser());
            result.setUrl("/");

            AuthUtil.setOrganize(user.getUser().getOrganize());

            return result;
        };
    }

    private boolean validateHost(HttpServletRequest request) {
        final String delimiter = ",";
        String hostsConfig = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.SSO_REQUEST_HOST);
        String[] hosts = hostsConfig.split(delimiter);
        String requestHost = ServletRequestTool.getRequestHost(request);
        log.info("{}地址正在尝试访问 E9 SSO", requestHost);
        return Arrays.stream(hosts).anyMatch(d -> requestHost.equals(d));
    }
}
